@@ -1,5 +1,5 @@
 from .exceptions import AccessPolicyException
-from .access_policy import AccessPolicy
+from .access_policy import AccessPolicy, Statement
 from .access_view_set_mixin import AccessViewSetMixin
 from .field_access_mixin import FieldAccessMixin
 from .fields import PermittedPkRelatedField, PermittedSlugRelatedField
